// $Id$

/*!
 * \file kat_aux.h
 * \brief Header file for auxilliary routines in Finesse
 * 
 * @section Copyright notice
 *
 *  This file is part of the interferometer simulation Finesse
 *  http://www.gwoptics.org/finesse
 *
 *  Copyright (C) 1999 onwards Andreas Freise
 *  with parts of the code written by Daniel Brown, Paul Cochrane
 *  and Gerhard Heinzel.
 *
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License version 3 as published
 *  by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this library; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 *  Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef KAT_AUX_H
#define KAT_AUX_H

void get_path_io(const char* input, char* output);
const char* get_path_i(const char* input);
int getNumCores();
int get_signal_type(char *s);
char *get_signal_type_str(int signal_type);
void check_for_long_line(char *s);
int prepare_line(char *s, int mode);
void check_funcname(char *name, const char *s);
int find_equal(char *name, char *fstring);
void sort_set(void);
void sort_variable_names(void);
void insert_constants_values(char *s);
int replace_string(char *s, char *s1, char *s2, int s1len, int s2len);
int get_mirror_index_from_name(int mirror_index, char *mapname);
int get_field_index_from_tem(int n, int m);
int get_field_index_from_local_tem(int n, int m, int maxtem);
void set_all_tems(void);
//void tem(int *n, int *m, int index);
double **dmatrix(int n, int m);
complex_t **cmatrix(int n, int m);
int **nmatrix(int n, int m);
double fac(int n);
void print_time(double secs);
void sprint_time(char *message_str, double secs);
double xdb(double x);
double hermite(int n, double x);
double dbesj0(double x);
double dbesj1(double x);
double bessn(int n, double x);
void check_name(const char *s);
int find_name(const char *s);
void dupdump(void); //!< Dump duplicate string
char *duplicate_string(const char *s);
char *duplicate_to_longer_string(const char *s, int n);
int get_node_index_for(const char *node_name);
int update_node_index_for(const char *node_name);
int get_reduced_index_for(int node_index);
int get_overall_component_index(int type, int component_index);
int get_function(double **function, const char *input_string);
int set_nr(const char *input_string);
int get_component_index_from_name(const char *s);
int get_component_type(int i);
int get_component_index(int i);
int get_component_type_decriment_index(int *i);
char *get_component_name(const int i);
char *get_component_type_name(const int i);
int get_node_index_from_name(const char *node_name);
int get_detector_or_node_index_from_name(const char *s);
char *get_detector_name(const int i);
int ctoi(char c, int *i);
int my_atoi(char *s, int *i);
bool atod(char *s, double *d);
char *complex_form(const complex_t z);
char *complex_form15(const complex_t z);
char *freq_form(double f);
char *int_form(int i);
char *double_form(double x);
char *xdouble_form(double x);
char *long_form(double c, int space);
void si_prefix(double *d, char *s);
complex_t my_cpow(complex_t z, double x);
complex_t my_cexp(complex_t z);
complex_t my_clog(complex_t z);
complex_t pow_complex(complex_t z, int nom, int denom);
double zeroin(double ax, double bx, double (*f) (double), double tol);
bool string_matches(char *input_string, char *match_string);

#endif // KAT_AUX_H

/*
 * Local variables:
 * c-indentation-style: bsd
 * c-basic-offset: 2
 * indent-tabs-mode: nil
 * End:
 *
 * vim: tabstop=2 expandtab shiftwidth=2:
 */
