// $Id$

/*!
 * \file kat_spa.h
 * \brief Header file for sparse matrix routines in Finesse
 * 
 * @section Copyright notice
 *
 *  This file is part of the interferometer simulation Finesse
 *  http://www.gwoptics.org/finesse
 *
 *  Copyright (C) 1999 onwards Andreas Freise
 *  with parts of the code written by Daniel Brown, Paul Cochrane
 *  and Gerhard Heinzel.
 *
 *  This program is free software; you can redistribute it and/or modify it under
 *  the terms of the GNU General Public License version 3 as published
 *  by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  this library; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 *  Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef KAT_SPA_H
#define KAT_SPA_H

//! format string for matrix formatting statements
#define MAT_FMT "%5.2g" 

//! format string for matrix formatting statements
#define SPA_FMT "%4.2g" 

//! the format string to use in fprintf statements
#define SOLN_FMT "%4.2g" 

//! the format string to use in fprintf statements
#define RHS_FMT "%4.2g" 

complex_t *get_matrix_elem(int i, int j, int matrix_type);
complex_t *get_sparse_matrix_elem(int i, int j, int matrix_type);

void spa_sol(int type);
void spa_tsol(int type);
void spa_fill(double f, int type);
void spa_rhs(int vector_index, complex_t z_value, int type);
void klu_rhs(int vector_index, complex_t z_value, int type);
void sparse_rhs(int vector_index, complex_t z_value, int type);
void spa_rhs_clear(int type);
void spa_sparse_rhs_clear(int type);
void spa_klu_rhs_clear(int type);
void spa_factor(int type);
void spa_sparse_factor(int type);
void spa_klu_factor(int type);
void dump_solution(void);
void dump_solution_vector(int type, char *output_stream);
void dump_rhs_vector(int type, char *output_stream);
void dump_matrix(int type, char *output_stream);
void dump_sparse_matrix(int type, char *output_stream);

void spa_create(void);
void spa_destroy(void);
void spa_get(void);

void check_connections(void);
void get_mirror_matrix_elements(int num_eqns_per_node, int mode);
void get_beamsplitter_matrix_elements(int num_eqns_per_node, int mode);
void get_modulator_matrix_elements(int num_eqns_per_node, int mode);
void get_space_matrix_elements(int num_eqns_per_node, int mode);
void get_squeezer_matrix_elements(int num_eqns_per_node, int mode);
void get_lens_matrix_elements(int num_eqns_per_node, int mode);
void get_diode_matrix_elements(int num_eqns_per_node, int mode);
void get_grating_matrix_elements(int num_eqns_per_node, int mode);

void init_sparse_matrix_diags(int type);
void fill_matrix_mirror_elements(double f);
void fill_matrix_beamsplitter_elements(double f);
void fill_matrix_grating_elements(void);
void fill_matrix_space_elements(double f);
void fill_matrix_squeezer_elements(void);
void fill_matrix_modulator_elements(void);
void fill_matrix_lens_elements(void);
void fill_matrix_diode_elements(void);

void dump_matrix_mirror_elements(FILE *fp);
void dump_matrix_beamsplitter_elements(FILE *fp);
void dump_matrix_space_elements(FILE *fp);

void report_sparse_matrix_error(int err, int pos);
void update_element_count(int node_index1,int node_index2, int col_index_1, int col_index_2, int mode);
void reset_connect_count(void);

#endif // KAT_SPA_H

/*
 * Local variables:
 * c-indentation-style: bsd
 * c-basic-offset: 2
 * indent-tabs-mode: nil
 * End:
 *
 * vim: tabstop=2 expandtab shiftwidth=2:
 */
